@@ -9,6 +9,26 @@
 #' @return
 #' The subtree, as an \code{\link[rpart]{rpart}} object.
 #'
+#' @examples
+#' ## Generate data.
+#' set.seed(1986)
+#'
+#' n <- 3000
+#' k <- 3
+#'
+#' X <- matrix(rnorm(n * k), ncol = k)
+#' colnames(X) <- paste0("x", seq_len(k))
+#'
+#' y <- exp(X[, 1]) + 2 * X[, 2] * X[, 2] > 0 + rnorm(n)
+#'
+#' ## Construct tree.
+#' library(rpart)
+#' tree <- rpart(y ~ ., data = data.frame(y, X), cp = 0)
+#'
+#' ## Extract subtree.
+#' sub_tree <- subtree(tree, leaves = 4)
+#' sub_tree_cv <- subtree(tree, cv = TRUE)
+#'
 #' @import rpart
 #'
 #' @author Riccardo Di Francesco
@@ -40,6 +60,26 @@ subtree <- function(tree, leaves = NULL, cv = FALSE) {
 #' @return
 #' The number of leaves.
 #'
+#' @examples
+#' ## Generate data.
+#' set.seed(1986)
+#'
+#' n <- 3000
+#' k <- 3
+#'
+#' X <- matrix(rnorm(n * k), ncol = k)
+#' colnames(X) <- paste0("x", seq_len(k))
+#'
+#' y <- exp(X[, 1]) + 2 * X[, 2] * X[, 2] > 0 + rnorm(n)
+#'
+#' ## Construct tree.
+#' library(rpart)
+#' tree <- rpart(y ~ ., data = data.frame(y, X))
+#'
+#' ## Extract number of leaves.
+#' n_leaves <- get_leaves(tree)
+#' n_leaves
+#'
 #' @author Riccardo Di Francesco
 #'
 #' @seealso \code{\link{subtree}} \code{\link{node_membership}} \code{\link{leaf_membership}}
@@ -64,6 +104,26 @@ get_leaves <- function(tree) {
 #' @return
 #' Logical vector denoting whether each observation in \code{X} falls into \code{node}.
 #'
+#' @examples
+#' ## Generate data.
+#' set.seed(1986)
+#'
+#' n <- 3000
+#' k <- 3
+#'
+#' X <- matrix(rnorm(n * k), ncol = k)
+#' colnames(X) <- paste0("x", seq_len(k))
+#'
+#' y <- exp(X[, 1]) + 2 * X[, 2] * X[, 2] > 0 + rnorm(n)
+#'
+#' ## Construct tree.
+#' library(rpart)
+#' tree <- rpart(y ~ ., data = data.frame(y, X))
+#'
+#' ## Extract number of leaves.
+#' is_in_third_node <- node_membership(tree, X, 3)
+#' is_in_third_node
+#'
 #' @importFrom utils capture.output
 #'
 #' @author Riccardo Di Francesco
@@ -90,6 +150,26 @@ node_membership <- function(tree, X, node) {                 # Taken from https:
 #' A factor whose levels denote in which leaf each unit falls. Leaves are ordered in increasing order of their predictions
 #' (from most negative to most positive).
 #'
+#' @examples
+#' ## Generate data.
+#' set.seed(1986)
+#'
+#' n <- 3000
+#' k <- 3
+#'
+#' X <- matrix(rnorm(n * k), ncol = k)
+#' colnames(X) <- paste0("x", seq_len(k))
+#'
+#' y <- exp(X[, 1]) + 2 * X[, 2] * X[, 2] > 0 + rnorm(n)
+#'
+#' ## Construct tree.
+#' library(rpart)
+#' tree <- rpart(y ~ ., data = data.frame(y, X))
+#'
+#' ## Extract number of leaves.
+#' leaves_factor <- leaf_membership(tree, X)
+#' leaves_factor
+#'
 #' @author Riccardo Di Francesco
 #'
 #' @seealso \code{\link{subtree}} \code{\link{node_membership}} \code{\link{get_leaves}}
@@ -108,9 +188,10 @@ leaf_membership <- function(tree, X) {
 }
 
 
-#' Estimation of rpart Objects
+#' GATE Estimation with rpart Objects
 #'
-#' Replace node predictions of an \code{\link[rpart]{rpart}} object using external data.
+#' Replaces node predictions of an \code{\link[rpart]{rpart}} object using external data to estimate the group average treatment
+#' effects (GATEs).
 #'
 #' @param tree An \code{\link[rpart]{rpart}} object.
 #' @param y Outcome vector.
@@ -123,13 +204,48 @@ leaf_membership <- function(tree, X) {
 #' A tree with node predictions replaced, as an \code{\link[rpart]{rpart}} object, and the scores (if \code{method == "raw"},
 #' this is \code{NULL}).
 #'
+#' @examples
+#' ## Generate data.
+#' set.seed(1986)
+#'
+#' n <- 1000
+#' k <- 3
+#'
+#' X <- matrix(rnorm(n * k), ncol = k)
+#' colnames(X) <- paste0("x", seq_len(k))
+#' D <- rbinom(n, size = 1, prob = 0.5)
+#' mu0 <- 0.5 * X[, 1]
+#' mu1 <- 0.5 * X[, 1] + X[, 2]
+#' y <- mu0 + D * (mu1 - mu0) + rnorm(n)
+#'
+#' ## Split the sample.
+#' splits <- sample_split(length(y), training_frac = 0.5)
+#' training_idx <- splits$training_idx
+#' honest_idx <- splits$honest_idx
+#'
+#' y_tr <- y[training_idx]
+#' D_tr <- D[training_idx]
+#' X_tr <- X[training_idx, ]
+#'
+#' y_hon <- y[honest_idx]
+#' D_hon <- D[honest_idx]
+#' X_hon <- X[honest_idx, ]
+#'
+#' ## Construct a tree using training sample.
+#' library(rpart)
+#' tree <- rpart(y ~ ., data = data.frame("y" = y_tr, X_tr), maxdepth = 2)
+#'
+#' ## Estimate GATEs in each node (internal and terminal) using honest sample.
+#' new_tree <- estimate_rpart(tree, y_hon, D_hon, X_hon, method = "raw")
+#' new_tree$tree
+#'
 #' @details
 #' If \code{method == "raw"}, \code{estimate_rpart} replaces node predictions with the differences between the sample average
 #' of the observed outcomes of treated units and the sample average of the observed outcomes of control units in each node,
-#' which is an unbiased estimator of GATEs if the assignment to treatment is randomized.\cr
+#' which is an unbiased estimator of the GATEs if the assignment to treatment is randomized.\cr
 #'
 #' If \code{method == "aipw"}, \code{estimate_rpart} replaces node predictions with sample averages of doubly-robust
-#' scores in each node. This is a valid estimator of GATEs in observational studies. Honest regression forests
+#' scores in each node. This is a valid estimator of the GATEs in observational studies. Honest regression forests
 #' and 5-fold cross fitting are used to estimate the propensity score and the conditional mean function of the outcome
 #' (unless the user specifies the argument \code{scores}).\cr
 #'
@@ -185,12 +301,10 @@ estimate_rpart <- function(tree, y, D, X, method = "aipw", scores = NULL) {
 }
 
 
-#' Estimating Leaf-Effects via Linear Models
+#' Estimation and Inference about the GATEs with rpart Objects
 #'
-#' Uses the leaves of a tree stored in an \code{\link[rpart]{rpart}} object to estimate a linear model via OLS. The
-#' estimated coefficients identify the GATE in each leaf. If the data used in the OLS estimation have not been
-#' used to grow the tree (a condition called "honesty"), then one can use the standard errors for the tree's estimates
-#' to construct valid confidence intervals.
+#' Obtains point estimates and standard errors for the group average treatment effects (GATEs), where groups correspond to the
+#' leaves of an \code{\link[rpart]{rpart}} object. Additionally, performs some hypothesis testing.
 #'
 #' @param tree An \code{\link[rpart]{rpart}} object.
 #' @param y Outcome vector.
@@ -200,37 +314,84 @@ estimate_rpart <- function(tree, y, D, X, method = "aipw", scores = NULL) {
 #' @param scores Optional, vector of scores to be used in the regression. Useful to save computational time if scores have already been estimated. Ignored if \code{method == "raw"}.
 #'
 #' @return
-#' The fitted model, as an \code{\link[estimatr]{lm_robust}} object, and the scores (if \code{method == "raw"}, this is
-#' \code{NULL}).
+#' A list storing:
+#'   \item{\code{model}}{The model fitted to get point estimates and standard errors for the GATEs, as an \code{\link[estimatr]{lm_robust}} object.}
+#'   \item{\code{gates_diff_pairs}}{Results of testing whether GATEs differ across all pairs of leaves. This is a list storing GATEs differences and p-values adjusted using Holm's procedure (check \code{\link[stats]{p.adjust}}). \code{NULL} if the tree consists of a root only.}
+#'   \item{\code{scores}}{Vector of doubly robust scores. \code{NULL} if \code{method == 'raw'}.}
 #'
+#' @examples
+#' ## Generate data.
+#' set.seed(1986)
+#'
+#' n <- 1000
+#' k <- 3
+#'
+#' X <- matrix(rnorm(n * k), ncol = k)
+#' colnames(X) <- paste0("x", seq_len(k))
+#' D <- rbinom(n, size = 1, prob = 0.5)
+#' mu0 <- 0.5 * X[, 1]
+#' mu1 <- 0.5 * X[, 1] + X[, 2]
+#' y <- mu0 + D * (mu1 - mu0) + rnorm(n)
+#'
+#' ## Split the sample.
+#' splits <- sample_split(length(y), training_frac = 0.5)
+#' training_idx <- splits$training_idx
+#' honest_idx <- splits$honest_idx
+#'
+#' y_tr <- y[training_idx]
+#' D_tr <- D[training_idx]
+#' X_tr <- X[training_idx, ]
+#'
+#' y_hon <- y[honest_idx]
+#' D_hon <- D[honest_idx]
+#' X_hon <- X[honest_idx, ]
+#'
+#' ## Construct a tree using training sample.
+#' library(rpart)
+#' tree <- rpart(y ~ ., data = data.frame("y" = y_tr, X_tr), maxdepth = 2)
+#'
+#' ## Estimate GATEs in each node (internal and terminal) using honest sample.
+#' results <- causal_ols_rpart(tree, y_hon, D_hon, X_hon, method = "raw")
+#'
+#' summary(results$model) # Coefficient of leafk:D is GATE in k-th leaf.
+#'
+#' results$gates_diff_pair$gates_diff # GATEs differences.
+#' results$gates_diff_pair$holm_pvalues # leaves 1-2 and 3-4 not statistically different.
+#'
+#' @md
 #' @details
-#' The \code{method} argument controls how GATEs are estimated. If \code{"method" == "raw"}, we estimate via OLS the following
-#' linear model:
+#' ## Point estimates and standard errors for the GATEs
+#' The GATEs and their standard errors are obtained by fitting an appropriate linear model. If \code{method == "raw"}, we
+#' estimate via OLS the following:
 #'
 #' \deqn{Y_i = \sum_{l = 1}^{|T|} L_{i, l} \gamma_l + \sum_{l = 1}^{|T|} L_{i, l} D_i \beta_l + \epsilon_i}
 #'
-#' with \code{L_{i, l}} a dummy variable equal to one if the i-th unit falls in the l-th leaf of the tree, and \code{|T|} the
-#' number of leaves. If the treatment is randomly assigned, one can show that the betas identify the GATE of each group.
-#' Thus, we can interpret the OLS results as usual. However, in observational studies these estimates are biased
-#' due to selection into treatment. To get unbiased estimates, we can set \code{"method"} to \code{"aipw"} to construct
-#' doubly-robust scores \code{y_i^*} and use them as a pseudo-outcome in the following regression:
+#' with \code{L_{i, l}} a dummy variable equal to one if the i-th unit falls in the l-th leaf of \code{tree}, and |T| the number of
+#' groups. If the treatment is randomly assigned, one can show that the betas identify the GATE in each leaf. However, this is not true
+#' in observational studies due to selection into treatment. In this case, the user is expected to use \code{method == "aipw"} to run
+#' the following regression:
 #'
-#' \deqn{Y_i^* = \sum_{l = 1}^{|T|} L_{i, l} \beta_l + \epsilon_i}
+#' \deqn{score_i = \sum_{l = 1}^{|T|} L_{i, l} \beta_l + \epsilon_i}
 #'
-#' This way, we get unbiased GATEs estimates, and we can again interpret OLS results as usual. Honest regression forests
-#' and 5-fold cross fitting are used to estimate the propensity score and the conditional mean function of the outcome
-#' (unless the user specifies the argument \code{scores}).\cr
-#'
-#' Notice that "honesty" is a necessary requirement to get valid inference. Thus, observations in \code{y}, \code{D}, and
-#' \code{X} must not have been used to grow the \code{tree}.\cr
+#' where score_i are doubly-robust scores constructed via honest regression forests and 5-fold cross fitting (unless the user specifies
+#' the argument \code{scores}). This way, betas again identify the GATEs.\cr
 #'
 #' Regardless of \code{method}, standard errors are estimated via the Eicker-Huber-White estimator.\cr
 #'
 #' If \code{tree} consists of a root only, \code{causal_ols_rpart} regresses \code{y} on a constant and \code{D} if
 #' \code{method == "raw"}, or regresses the doubly-robust scores on a constant if \code{method == "aipw"}. This way,
-#' we get an estimate of the ATE.
+#' we get an estimate of the overall average treatment effect.
 #'
-#' @import rpart estimatr
+#' ## Hypothesis testing
+#' \code{\link{causal_ols_rpart}} uses the standard errors obtained by fitting the linear models above to test the hypotheses
+#' that the GATEs are different across all pairs of leaves. Here, we adjust p-values to account for multiple hypotheses testing
+#' using Holm's procedure.
+#'
+#' ## Caution on Inference
+#' "honesty" is a necessary requirement to get valid inference. Thus, observations in \code{y}, \code{D}, and
+#' \code{X} must not have been used to construct the \code{tree} and the \code{scores}.\cr
+#'
+#' @import rpart estimatr car stats
 #'
 #' @author Riccardo Di Francesco
 #'
@@ -242,16 +403,16 @@ estimate_rpart <- function(tree, y, D, X, method = "aipw", scores = NULL) {
 #' @seealso \code{\link{estimate_rpart}} \code{\link{avg_characteristics_rpart}}
 #'
 #' @export
-causal_ols_rpart <- function(tree, y, X, D, method = "aipw", scores = NULL) {
+causal_ols_rpart <- function(tree, y, D, X, method = "aipw", scores = NULL) {
   if (!inherits(tree, "rpart")) stop("'tree' must be a rpart object.", call. = FALSE)
   if(!(method %in% c("raw", "aipw"))) stop("Invalid 'method'. It must be either 'raw' or 'aipw'.", call. = FALSE)
 
   ## Generate leaves indicators.
   leaves <- leaf_membership(tree, X)
+  n_leaves <- get_leaves(tree)
+  if (length(unique(leaves)) < n_leaves) warning("One or more leaves are empty: No observations in X fall there.")
 
-  if (length(unique(leaves)) < get_leaves(tree)) warning("One or more leaves are empty: No observations in X fall there.")
-
-  ## OLS estimation.
+  ## GATEs point estimates and standard errors.
   if (method == "raw") {
     if (length(unique(leaves)) == 1) {
       model <- estimatr::lm_robust(y ~ D, data = data.frame("y" = y, "D" = D), se_type = "HC1")
@@ -270,31 +431,95 @@ causal_ols_rpart <- function(tree, y, X, D, method = "aipw", scores = NULL) {
     }
   }
 
+  ## Test if GATEs are different across all pairs of leaves. Adjust p-values by Holm's procedure.
+  if (n_leaves > 1) { # Inspired by https://gsbdbi.github.io/ml_tutorial/hte_tutorial/hte_tutorial.html#hte_1:_causal_trees.
+    gates_point <- model$coefficients
+
+    differences <- matrix(NA, nrow = n_leaves, ncol = n_leaves)
+    rownames(differences) <- paste0("leaf", seq_len(n_leaves))
+    colnames(differences) <- paste0("leaf", seq_len(n_leaves))
+
+    p_values <- matrix(NA, nrow = n_leaves, ncol = n_leaves)
+    rownames(p_values) <- paste0("leaf", seq_len(n_leaves))
+    colnames(p_values) <- paste0("leaf", seq_len(n_leaves))
+
+    if (method == "raw") {
+      for (i in seq_len(n_leaves-1)) {
+        for (j in seq_len(n_leaves)[-c(1:i)]) {
+         leaf_i <- paste0("leaf", i, ":D")
+         leaf_j <- paste0("leaf", j, ":D")
+
+         differences[j, i] <- gates_point[leaf_j] - gates_point[leaf_i]
+         p_values[j, i] <- car::linearHypothesis(model, paste(leaf_j, "=", leaf_i), test = "F")$`Pr(>F)`[2]
+       }
+     }
+    } else if (method == "aipw") {
+      for (i in seq_len(n_leaves-1)) {
+        for (j in seq_len(n_leaves)[-c(1:i)]) {
+          leaf_i <- paste0("leaf", i)
+          leaf_j <- paste0("leaf", j)
+
+          differences[j, i] <- gates_point[leaf_j] - gates_point[leaf_i]
+          p_values[j, i] <- car::linearHypothesis(model, paste(leaf_j, "=", leaf_i), test = "F")$`Pr(>F)`[2]
+        }
+      }
+    }
+
+    p_values_vec <- c(p_values) # First column, then second columns, then third columns ...
+    p_values_holm_vec <- stats::p.adjust(p_values_vec, method = "holm")
+    p_values_holm <- matrix(p_values_holm_vec, nrow = n_leaves, ncol = n_leaves)
+
+    gates_diff_pairs <- list("gates_diff" = differences, "holm_pvalues" = p_values_holm)
+  } else {
+    gates_diff_pairs <- NULL
+  }
+
   ## Output.
-  return(list("model" = model, "scores" = scores))
+  return(list("model" = model,
+              "gates_diff_pairs" = gates_diff_pairs,
+              "scores" = scores))
 }
 
 
 #' Leaves Average Characteristics
 #'
-#' Compute the average characteristics of units in each leaf of an \code{\link[rpart]{rpart}} object.
+#' Computes the average characteristics of units in each leaf of an \code{\link[rpart]{rpart}} object.
 #'
 #' @param tree An \code{rpart} object.
 #' @param X Covariate matrix (no intercept).
-#' @param gates_point Estimated GATEs, one for each leaf, in increasing order (from most negative to most positive).
-#' @param gates_sd Standard errors for estimated GATEs, one for each leaf.
 #'
 #' @return
-#' Prints LATEX code in the console.
+#' A list storing each regression as an \code{\link[estimatr]{lm_robust}} object.
+#'
+#' @examples
+#' ## Generate data.
+#' set.seed(1986)
+#'
+#' n <- 1000
+#' k <- 3
+#'
+#' X <- matrix(rnorm(n * k), ncol = k)
+#' colnames(X) <- paste0("x", seq_len(k))
+#' D <- rbinom(n, size = 1, prob = 0.5)
+#' mu0 <- 0.5 * X[, 1]
+#' mu1 <- 0.5 * X[, 1] + X[, 2]
+#' y <- mu0 + D * (mu1 - mu0) + rnorm(n)
+#'
+#' ## Construct a tree.
+#' library(rpart)
+#' tree <- rpart(y ~ ., data = data.frame("y" = y, X), maxdepth = 2)
+#'
+#' ## Compute average characteristics in each leaf.
+#' results <- avg_characteristics_rpart(tree, X)
+#' results
 #'
 #' @details
-#' \code{avg_characteristics_rpart} regresses each covariate on a set of dummies denoting leaf membership.
-#' This way, we get the average characteristics of units in each leaf, together with a standard error. Leaves are
-#' ordered in increasing order of their predictions (from most negative to most positive). Standard errors are
-#' estimated via the Eicker-Huber-White estimator.\cr
+#' \code{\link{avg_characteristics_rpart}} regresses each covariate on a set of dummies denoting leaf membership.
+#' This way, we get the average characteristics of units in each leaf, together with a standard error.\cr
 #'
-#' Compilation of the LATEX code requires the following packages: \code{booktabs}, \code{float}, \code{adjustbox},
-#' \code{multirow}.
+#' Leaves are ordered in increasing order of their predictions (from most negative to most positive).\cr
+#'
+#' Standard errors are estimated via the Eicker-Huber-White estimator.
 #'
 #' @import estimatr stats
 #'
@@ -308,7 +533,7 @@ causal_ols_rpart <- function(tree, y, X, D, method = "aipw", scores = NULL) {
 #' @seealso \code{\link{causal_ols_rpart}}, \code{\link{estimate_rpart}}
 #'
 #' @export
-avg_characteristics_rpart <- function(tree, X, gates_point = NULL, gates_sd = NULL) {
+avg_characteristics_rpart <- function(tree, X) {
   ## Handling inputs and checks.
   if (!inherits(tree, "rpart")) stop("'tree' must be a rpart object.", call. = FALSE)
 
@@ -318,60 +543,6 @@ avg_characteristics_rpart <- function(tree, X, gates_point = NULL, gates_sd = NU
   ## Regress each leaf on the leaf indicator.
   regressions <- apply(X, MARGIN = 2, function(x) {estimatr::lm_robust(x ~ 0 + leaf, data = data.frame("x" = x, "leaf" = leaves), se_type = "HC1")})
 
-  ## Extract information.
-  parms <- lapply(regressions, function(x) {stats::coef(summary(x))[, c("Estimate", "Std. Error")]})
-  if (is.null(gates_point)) gates_point <- rep("NA", lenght = length(unique(leaves)))
-  if (is.null(gates_sd)) gates_sd <- rep("NA", lenght = length(unique(leaves)))
-
-  gates_point <- round(gates_point, 3)
-  gates_sd <- round(gates_sd, 3)
-  gates_ci_lower <- round(gates_point - 1.96 * gates_sd, 3)
-  gates_ci_upper <- round(gates_point + 1.96 * gates_sd, 3)
-
-  ## Write table.
-  table_names <- rename_latex(colnames(X))
-
-  cat("\\begingroup
-  \\setlength{\\tabcolsep}{8pt}
-  \\renewcommand{\\arraystretch}{1.1}
-  \\begin{table}[b!]
-    \\centering
-    \\begin{adjustbox}{width = 1\\textwidth}
-    \\begin{tabular}{@{\\extracolsep{5pt}}l ", rep("c ", length(unique(leaves))), "}
-      \\\\[-1.8ex]\\hline
-      \\hline \\\\[-1.8ex]
-      & ", c(paste("\\textit{Leaf ", 1:(length(unique(leaves))-1), "} & ", sep = ""), paste("\\textit{Leaf ", length(unique(leaves)), "}", sep = "")) ," \\\\
-      \\addlinespace[2pt]
-      \\hline \\\\[-1.8ex] \n\n", sep = "")
-
-  cat("      \\multirow{2}{*}{GATEs} & ", paste(gates_point[1:(length(unique(leaves))-1)], " & ", sep = ""), gates_point[length(unique(leaves))], " \\\\
-      & ", paste("[", gates_ci_lower[1:(length(unique(leaves))-1)], ", ", gates_ci_upper[1:(length(unique(leaves))-1)], "] & ", sep = ""), paste("[", gates_ci_lower[length(unique(leaves))], ", ", gates_ci_upper[length(unique(leaves))], "]", sep = ""), " \\\\ \n\n", sep = "")
-  cat("      \\addlinespace[2pt]
-      \\hline \\\\[-1.8ex] \n\n")
-
-  for (i in seq_len(length(table_names))) {
-    cat("      \\texttt{", table_names[i], "} & ", paste(round(parms[[i]][1:(length(unique(leaves))-1), 1], 2), " & ", sep = ""), round(parms[[i]][length(unique(leaves)), 1], 2), " \\\\ \n",
-        "      & ", paste("(", round(parms[[i]][1:(length(unique(leaves))-1), 2], 3), ")", " & ", sep = ""), paste("(", round(parms[[i]][length(unique(leaves)), 2], 3), ")", sep = ""), " \\\\ \n", sep = "")
-  }
-
-  cat("\n      \\addlinespace[3pt]
-      \\\\[-1.8ex]\\hline
-      \\hline \\\\[-1.8ex]
-    \\end{tabular}
-    \\end{adjustbox}
-    \\caption{\\href{https://soundcloud.com/theplasticchairband}{Secret link.}}
-    \\label{table:average.characteristics.leaves}
-    \\end{table}
-\\endgroup \n\n")
-
-  no_variation_names <- names(unlist(lapply(parms, function(x) { if (any(x[, 2] == 0)) idx <- 1})))
-
-  ## Warn the user for zero variation in leaves.
-  if (length(no_variation_names) > 1) {
-    names_string <- paste0(no_variation_names, collapse = ", ")
-    warning(paste0("Variables '", names_string, "' have no variation in one or more leaves. Please correct the table by removing the associated standard errors."))
-  } else if (length(no_variation_names) == 1) {
-    warning(paste0("Variable '", no_variation_names, "' has no variation in one or more leaves. Please correct the table by removing the associated standard errors."))
-
-  }
+  ## Output.
+  return(regressions)
 }
